/*
  Desktop Station Empty Box
  Copyright(C) '2016- Desktop Station / yaasan

  The circuit:
 * LCD RS pin to digital pin 9
 * LCD Enable pin to digital pin 8
 * LCD D4 pin to digital pin 13
 * LCD D5 pin to digital pin 12
 * LCD D6 pin to digital pin 11
 * LCD D7 pin to digital pin 10
 * LCD R/W pin to ground
 * LCD VSS pin to ground
 * LCD VCC pin to 5V
 * 10K resistor:
 * ends to +5V and ground
 * wiper to LCD VO pin (pin 3)

 This example code is in the public domain.

 http://www.arduino.cc/en/Tutorial/LiquidCrystal
 */

// include the library code:
#include <LiquidCrystal.h>
#include <EEPROM.h>
#include <avr/wdt.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "DSCoreM_Type.h"
#include "DSCoreM_Common.h"
#include "DSCoreM_List.h"
#include "DSCoreM_DCC.h"
#include "DSCoreM_MM2.h"
#include "DSCoreM.h"
#include "DSSequence.h"



// initialize the library with the numbers of the interface pins
LiquidCrystal lcd(9, 8, 13, 12, 11, 10);

DSSequence Sequence;
DSCoreLib DSCore;

//Task Schedule
unsigned long gPreviousL5 = 0; //   500ms interval
unsigned long gPreviousL1 = 0; //   100ms interval

//LCD text
char gTextBuf[16 + 1];
byte gDisplayUpdate = 0;

void displayCVRead(byte inNo);
void displayCVReadWrite(byte inNo);
void displayAddrCheck(byte inNo);
void displayAddrWrite(byte inNo);
void displayNuckySignal(byte inNo);
void ExecuteCommand(unsigned char *outpReturn);


void setup() {
	
	DSCore.Init();
	
	// set up the LCD's number of columns and rows:
	lcd.begin(16, 2);
	// Print a message to the LCD.
	lcd.print("Desktop Station");
	lcd.setCursor(0, 1);
	lcd.print("CV Read&Writer");
	
	
	Serial.begin(115200);
	while (!Serial);

	Serial.println("Desktop Station Empty Box 0.2");
	Serial.println("2016 June 17");
	
	delay(1000);
	
	//DCCs֐
	Sequence.CVExeFunc = ExecuteCommand;

	
	/* EHb`hbOL(8sec) */
	wdt_enable(WDTO_8S); // WDT reset
	
	
	lcd.clear();
	
	
	gPreviousL5 = millis();
	gPreviousL1 = millis();
	sei();    //ENABLE INTERRUPTION
}

void loop() {
	
	
	/* Reset watchdog timer. */
	wdt_reset(); 
	
	//Button monitoring interval
	if( (millis() - gPreviousL1) >= 100)
	{
		
		gDisplayUpdate = Sequence.Interval();
		
		//Reset task
		gPreviousL1 = millis();
	}
	
	// LCD update interval
	if( ( (millis() - gPreviousL5) >= 500) || ( gDisplayUpdate > 0))
	{
		gDisplayUpdate = 0;
		Draw();
		
		
		//Reset task
		gPreviousL5 = millis();
	}
	
}



void Draw()
{
	lcd.clear();
	
	switch(Sequence.gMode)
	{
	case MODE_MAINMENU:
		
		for( int i = 0; i <= 1; i++)
		{
			lcd.setCursor(0, i);
			displayMainMenu(i + Sequence.gNo_MainMenu, i);
			
		}
		
		break;
		
	case MODE_CVREAD:
		
		displayCVRead(Sequence.gNo_CVReadMenu);
		
		
		break;
		
	case MODE_CVWRITE:
		
		displayCVWrite(Sequence.gNo_CVWriteMenu);
		
		
		break;
		
	case MODE_CVREADWRITE:
		
		displayCVReadWrite(Sequence.gNo_CVReadWriteMenu);
		
		
		break;
	
	case MODE_CHECKADDRESS:
		displayAddrCheck(Sequence.gNo_AddrChkMenu);
		
		break;
		
	case MODE_WRITEADDRESS:
		displayAddrWrite(Sequence.gNo_AddrWriteMenu);
		
		break;
		
	case MODE_NUCKYSIGNAL:
		displayNuckySignal(Sequence.gNo_NSignalMenu);
		
		break;
		
	}
	
	
	
}

void displayCVRead(byte inNo)
{
	char aTextBuf[16 + 1] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	
	//1s
	sprintf(aTextBuf, " CV No.  %04d", Sequence.CVNo);
	
	if( inNo == 0)
	{
		aTextBuf[0] = 0x3E;
	}
	lcd.setCursor(0, 0);
	lcd.print(aTextBuf);
	
	//Next row
	lcd.setCursor(0, 1);
	
	if( inNo == 1)
	{
	
		//2s
		sprintf(aTextBuf, " CV Reading...");
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
	
	}
	else if( inNo == 2)
	{
	
		//2s
		sprintf(aTextBuf, " CV Value %03d", Sequence.CVValue);
		
		aTextBuf[0] = 0x3E;
		
		lcd.print(aTextBuf);
	
	}
	else if (inNo == 3)
	{
		
		sprintf(aTextBuf, " Read error!");
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
		
	}
	
}

void displayCVWrite(byte inNo)
{
	char aTextBuf[16 + 1] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	
	//1s
	sprintf(aTextBuf, " CV No.  %04d", Sequence.CVNo);
	
	if( inNo == 0)
	{
		aTextBuf[0] = 0x3E;
	}
	lcd.setCursor(0, 0);
	lcd.print(aTextBuf);
	
	//Next row
	lcd.setCursor(0, 1);
	
	if( inNo <= 1)
	{
	
		//2s
		sprintf(aTextBuf, " CV Value %03d", Sequence.CVValue);
		
		if( inNo == 1)
		{
			aTextBuf[0] = 0x3E;
		}
		
		lcd.print(aTextBuf);
	
	}
	else if( inNo == 2)
	{
		
		sprintf(aTextBuf, " CV Writing...");
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
	}
	else if (inNo == 3)
	{
		
		sprintf(aTextBuf, " Write OK!");
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
		
	}
	else if (inNo == 4)
	{
		
		sprintf(aTextBuf, " Write error!");
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
		
	}
	
}


void displayCVReadWrite(byte inNo)
{
	char aTextBuf[16 + 1] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	
	//1s
	sprintf(aTextBuf, " CV No.  %04d", Sequence.CVNo);
	
	if( inNo == 0)
	{
		aTextBuf[0] = 0x3E;
	}
	lcd.setCursor(0, 0);
	lcd.print(aTextBuf);
	
	//Next row
	lcd.setCursor(0, 1);
	
	if( inNo == 1)
	{
	
		//2s
		sprintf(aTextBuf, " CV Reading...");
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
	
	}
	else if( inNo == 2)
	{
	
		//2s
		sprintf(aTextBuf, " CV Value %03d", Sequence.CVValue);
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
	
	}
	else if( inNo == 3)
	{
		
		sprintf(aTextBuf, " CV Writing...");
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
	}
	else if (inNo == 4)
	{
		
		sprintf(aTextBuf, " Write OK!");
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
		
	}
	else if (inNo == 5)
	{
		
		sprintf(aTextBuf, " Write error!");
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
		
	}
	else if (inNo == 6)
	{
		
		sprintf(aTextBuf, " Read error!");
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
		
	}
}


void displayAddrCheck(byte inNo)
{
	char aTextBuf[16 + 1] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	
	//1s
	sprintf(aTextBuf, " LocAddr Check");
	lcd.setCursor(0, 0);
	lcd.print(aTextBuf);
	
	//Next row
	lcd.setCursor(0, 1);
	
	if( inNo == 0)
	{
	
		//2s
		sprintf(aTextBuf, " Enter to start");
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
	
	}
	else if( inNo == 1)
	{
	
		//2s
		sprintf(aTextBuf, " CV Reading...");
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
	
	}
	else if( inNo == 2)
	{
	
		//2s
		sprintf(aTextBuf, " %04d(%s)", Sequence.Address, (Sequence.Address > 100) ? "LONG" : "SHORT");
		
		aTextBuf[0] = 0x3E;
		
		lcd.print(aTextBuf);
	
	}
	else if (inNo == 99)
	{
		
		sprintf(aTextBuf, " Read error!");
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
		
	}
	
}


void displayAddrWrite(byte inNo)
{
	char aTextBuf[16 + 1] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	
	//1s
	sprintf(aTextBuf, " LocAddr Write");
	lcd.setCursor(0, 0);
	lcd.print(aTextBuf);
	
	//Next row
	lcd.setCursor(0, 1);
	
	if( inNo == 0)
	{
	
		//2s
		sprintf(aTextBuf, " %04d", Sequence.Address);
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
	
	}
	else if( inNo == 1)
	{
	
		//2s
		sprintf(aTextBuf, " CV Reading...");
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
	
	}
	else if( inNo == 2)
	{
	
		//2s
		sprintf(aTextBuf, " CV Writing...");
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
	
	}
	else if( inNo == 3)
	{
	
		//2s
		sprintf(aTextBuf, " Finished!");
		
		aTextBuf[0] = 0x3E;
		
		lcd.print(aTextBuf);
	
	}
	else if( inNo == 4)
	{
	
		//2s
		sprintf(aTextBuf, " Failed!");
		
		aTextBuf[0] = 0x3E;
		
		lcd.print(aTextBuf);
	
	}
	else if (inNo == 99)
	{
		
		sprintf(aTextBuf, " Read error!");
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
		
	}
	
}

void displayNuckySignal(byte inNo)
{
	char aTextBuf[16 + 1] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	
	if( inNo <= 4)
	{
	
		//1s
		sprintf(aTextBuf, " NSig A%03d B%03d", Sequence.SignalAddr[0], Sequence.SignalAddr[1]);
		
		switch(inNo)
		{
			case 0: aTextBuf[5] = 0x3E; break;
			case 1: aTextBuf[10] = 0x3E; break;
			case 2: break;
			case 3: break;
			case 4: break;
		}
		
		lcd.setCursor(0, 0);
		lcd.print(aTextBuf);
		
		//Clear
		for( int i = 0; i < 16; i++)
		{
			aTextBuf[i] = 0;
		}
		
		
		//2s
		sprintf(aTextBuf, " C%03d D%03d E%03d", Sequence.SignalAddr[2], Sequence.SignalAddr[3], Sequence.SignalAddr[4]);
		
		switch(inNo)
		{
			case 0: break;
			case 1: break;
			case 2: aTextBuf[0] = 0x3E;break;
			case 3: aTextBuf[5] = 0x3E;break;
			case 4: aTextBuf[10] = 0x3E;break;
		}
		
		//Next row
		lcd.setCursor(0, 1);
		lcd.print(aTextBuf);
	
	}
	else
	{
		sprintf(aTextBuf, " Nucky Signal");
		lcd.setCursor(0, 0);
		lcd.print(aTextBuf);
		
		if( inNo == 5)
		{
			sprintf(aTextBuf, " CV Writing...");
			
		}
		else if (inNo == 6)
		{
			sprintf(aTextBuf, " Write Ok!");
			
		}
		else if (inNo == 99)
		{
			sprintf(aTextBuf, " Write End!");
			
		}

		//Next row
		lcd.setCursor(0, 1);
		aTextBuf[0] = 0x3E;
		lcd.print(aTextBuf);
		
	}
}


void displayMainMenu(byte inNo, byte inArrow)
{
	char aTextBuf[16 + 1] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	
	switch(inNo)
	{
	case 0:
		sprintf(aTextBuf, " CV Read");
		break;
	case 1:
		sprintf(aTextBuf, " CV Write");
		break;
	case 2:
		sprintf(aTextBuf, " CV ReadWrite");
		break;
	case 3:
		sprintf(aTextBuf, " Nucky Signal");
		break;
	case 4:
		sprintf(aTextBuf, " Check LocAddr");
		break;
	case 5:
		sprintf(aTextBuf, " Write LocAddr");
		break;
	}
	
	if( inArrow == 0)
	{
		aTextBuf[0] = 0x3E;
	}
	
	lcd.print(aTextBuf);
	
	
}


//

void ExecuteCommand(unsigned char *outpReturn)
{
	unsigned char aCVValue;
	
	PackedDScmd aDScmd = Sequence.GetCommad();
	
	Draw();
	
	switch(aDScmd.mCmd)
	{
	case DSCMD_CVREAD:
		
		aCVValue = 0;
		
		if( DSCore.ReadConfig(ADDR_DCC, aDScmd.mParam1, &aCVValue))
		{
			//Success
			Sequence.CVValue = aCVValue;
			*outpReturn = 2;
			
		}
		else
		{
			//Error
			Sequence.CVValue = 0;
			*outpReturn = 3;
			
		}
		break;
		
	case DSCMD_CVWRITE:
		if( DSCore.WriteConfig_Dir( aDScmd.mParam1, aDScmd.mParam2))
		{
			*outpReturn = 3;
		}
		else
		{
			//Error
			*outpReturn = 4;
		}
			
		break;
	/*
	case DSCMD_SPEED:
	case DSCMD_DIR:
		break;
	case DSCMD_NONE:
		break;
	*/
	}
	
}


